%DOLPHCHEB
% Function that computes Dolph-Chebyshev windowing function. The main role
% of the window (as with all windows) is to damp out the effects of the
% Gibbs phenomenon that results from truncation of an infinite series (e.g.
% a Discrete Fourier Transform). Note this window is complex
% and can therefore only really be visualised when the amplitude of the 
% window DFT is plotted (in dB).
%
% LAST UPDATED by Andrew French, AMS Ltd. 27/08/2004.
%
% Syntax: w=dolphcheb(N,sl_level)
%
% N      - Length of window sl_level. This ifference between peak and
%          sidelobes /dB in the DFT of the window.
% w      - Window 'weights'. These are real, positive numbers.

function w=dolphcheb(N,sl_level)
gamma=sl_level/20;
if N>1
    M=N-1;
    alpha=cosh(acosh(10^gamma)/M);
    for n=1:M
        A(n)=((-1)^(n-1))*cos(M*acos(alpha*cos(pi*(n-1)/M)));
    end
    B=fft(A);
    B(1)=0.5*B(1);
    B(N)=B(1); 
    w=B/max(B);
else
    w=1;    
end

%End of code